/*
 * Name:    mxcaMotorRecord.h
 *
 * Purpose: This header file defines the mxcaMotorRecord class.
 *
 *-----------------------------------------------------------------------
 *
 * Copyright 2004, 2009 Illinois Institute of Technology
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

extern "C" {
#include "mx_motor.h"
}

#define MXCA_MOTOR_FOFF_VARIABLE	0
#define MXCA_MOTOR_FOFF_FROZEN		1

#define MXCA_MOTOR_USE_MODE		0
#define MXCA_MOTOR_SET_MODE		1

#define MXCA_MOTOR_SPMG_STOP		0
#define MXCA_MOTOR_SPMG_PAUSE		1
#define MXCA_MOTOR_SPMG_MOVE		2
#define MXCA_MOTOR_SPMG_GO		3

class mxcaMotorDialPV;
class mxcaMotorDiffPV;
class mxcaMotorDirPV;
class mxcaMotorDvalPV;
class mxcaMotorHomePV;
class mxcaMotorJogPV;
class mxcaMotorMstaPV;
class mxcaMotorOffPV;
class mxcaMotorRlvPV;
class mxcaMotorRrbvPV;
class mxcaMotorRxpPV;
class mxcaMotorSpmgPV;
class mxcaMotorTweakPV;
class mxcaMotorUserPV;
class mxcaMotorValPV;

class mxcaMotorRecord : public mxcaRecord {
    friend class mxcaMotorDialPV;
    friend class mxcaMotorDiffPV;
    friend class mxcaMotorDirPV;
    friend class mxcaMotorDvalPV;
    friend class mxcaMotorHomePV;
    friend class mxcaMotorJogPV;
    friend class mxcaMotorMstaPV;
    friend class mxcaMotorOffPV;
    friend class mxcaMotorRlvPV;
    friend class mxcaMotorRrbvPV;
    friend class mxcaMotorSpmgPV;
    friend class mxcaMotorTweakPV;
    friend class mxcaMotorUserPV;
    friend class mxcaMotorValPV;

    public:
    	mxcaMotorRecord ( mxcaServer *epics_server,
			const char *epics_record_name,
			MX_RECORD *mx_record );

    private:
	void set_dval_to_drbv( void );

	mxcaPV          *accl_pv;
	mxcaPV          *athm_pv;
	mxcaPV          *bacc_pv;
	mxcaPV          *bdst_pv;
	mxcaPV          *bvel_pv;
	mxcaDummyPV     *card_pv;
	mxcaPV          *dcof_pv;
	mxcaPV          *desc_pv;
	mxcaMotorDialPV *dhlm_pv;
	mxcaMotorDiffPV *diff_pv;
	mxcaMotorDirPV  *dir_pv;
	mxcaMotorDialPV *dllm_pv;
	mxcaPV          *dmov_pv;
	mxcaMotorDialPV *drbv_pv;
	mxcaPV          *dtyp_pv;
	mxcaMotorDvalPV *dval_pv;
	mxcaPV          *egu_pv;
	mxcaPV          *eres_pv;
	mxcaEnumPV      *foff_pv;
	mxcaMotorUserPV *hlm_pv;
	mxcaPV          *hls_pv;
	mxcaMotorHomePV *homf_pv;
	mxcaMotorHomePV *homr_pv;
	mxcaPV          *icof_pv;
	mxcaMotorJogPV  *jogf_pv;
	mxcaMotorJogPV  *jogr_pv;
	mxcaMotorUserPV *llm_pv;
	mxcaPV          *lls_pv;
	mxcaPV          *lvio_pv;
	mxcaPV          *miss_pv;
	mxcaPV          *movn_pv;
	mxcaPV          *mres_pv;
	mxcaMotorMstaPV *msta_pv;
	mxcaPV          *name_pv;
	mxcaMotorOffPV  *off_pv;
	mxcaPV          *pcof_pv;
	mxcaMotorUserPV *rbv_pv;
	mxcaMotorRxpPV  *rep_pv;
	mxcaMotorRlvPV  *rlv_pv;
	mxcaMotorRxpPV  *rmp_pv;
	mxcaMotorRrbvPV *rrbv_pv;
	mxcaPV          *rval_pv;
	mxcaEnumPV      *set_pv;
	mxcaMotorSpmgPV *spmg_pv;
	mxcaPV          *stat_pv;
	mxcaPV          *stop_pv;
	mxcaPV          *tdir_pv;
	mxcaMotorTweakPV *twf_pv;
	mxcaMotorTweakPV *twr_pv;
	mxcaPV          *twv_pv;
	mxcaEnumPV      *ueip_pv;
	mxcaMotorValPV  *val_pv;
	mxcaPV          *vbas_pv;
	mxcaPV          *velo_pv;
	mxcaPV          *vers_pv;

	short         *athm_value;
	double        *dhlm_value;
	aitEnum16     *dir_enum_value;
	double        *dllm_value;
	short         *dmov_value;
	double        *dval_value;
	aitEnum16     *foff_value;
	short         *lvio_value;
	short         *movn_value;
	unsigned long *msta_value;
	double        *off_value;
	aitEnum16     *set_value;
	aitEnum16     *spmg_value;
	short         *tdir_value;
	double        *twv_value;
	double        *val_value;
};

class mxcaMotorDvalPV : public mxcaPV {
    friend class mxcaMotorRecord;
    friend class mxcaMotorValPV;

    public:
	mxcaMotorDvalPV( mxcaRecord *epics_record );

	caStatus read( void );
	caStatus write( void );
	mx_bool_type post_value_changed( void );

    private:
	MX_RECORD_FIELD *set_position_rf;
};

class mxcaMotorValPV : public mxcaPV {
    friend class mxcaMotorRecord;

    public:
	mxcaMotorValPV( mxcaRecord *epics_record );

	caStatus read( void );
	caStatus write( void );

    private:
	double val_value;
};

class mxcaMotorRlvPV : public mxcaPV {
    friend class mxcaMotorRecord;

    public:
	mxcaMotorRlvPV( mxcaRecord *epics_record );

	caStatus read( void );
	caStatus write( const gdd &gdd_object );

    private:
	double rlv_value;
};

class mxcaMotorDialPV : public mxcaPV {
    friend class mxcaMotorUserPV;
    friend class mxcaMotorDirPV;
    friend class mxcaMotorOffPV;
    friend class mxcaMotorRrbvPV;

    public:
	mxcaMotorDialPV( mxcaRecord *epics_record,
			const char *epics_field_name,
			const char *mx_field_name);

	caStatus read( void );
	caStatus write( void );
	mx_bool_type post_value_changed( void );

    private:
	double *dial_value;
	double *user_value;

	mxcaMotorUserPV *user_pv;
};

class mxcaMotorUserPV : public mxcaPV {
    friend class mxcaMotorDialPV;
    friend class mxcaMotorDirPV;
    friend class mxcaMotorOffPV;

    public:
	mxcaMotorUserPV( mxcaRecord *epics_record,
			const char *epics_field_name,
			mxcaMotorDialPV *dial_pv );

	caStatus read( void );
	caStatus write( void );

    private:
	double *dial_value;
	double *user_value;

	mxcaMotorDialPV *dial_pv;
};

class mxcaMotorRrbvPV : public mxcaPV {
    public:
	mxcaMotorRrbvPV( mxcaRecord *epics_record );

	mx_bool_type post_value_changed( void );
};

class mxcaMotorDirPV : public mxcaEnumPV {
    public:
	mxcaMotorDirPV( mxcaRecord *epics_record );

	caStatus write( const gdd &gdd_object );

	mx_bool_type post_value_changed( void );
};

class mxcaMotorOffPV : public mxcaPV {
    public:
	mxcaMotorOffPV( mxcaRecord *epics_record );

	caStatus write( void );
	mx_bool_type post_value_changed( void );
};

class mxcaMotorDiffPV : public mxcaPV {
    public:
	mxcaMotorDiffPV( mxcaRecord *epics_record );

	caStatus read( void );

    private:
	double deadband;
	double old_difference;
};

class mxcaMotorSpmgPV : public mxcaEnumPV {
    public:
	mxcaMotorSpmgPV( mxcaRecord *epics_record );

	caStatus write( const gdd &gdd_object );
};

class mxcaMotorJogPV : public mxcaPV {
    public:
	mxcaMotorJogPV( mxcaRecord *epics_record,
			const char *epics_field_name,
			int direction );

	caStatus write( const gdd &gdd_object );

    private:
	int direction;
	int jog_value;
};

class mxcaMotorHomePV : public mxcaPV {
    public:
	mxcaMotorHomePV( mxcaRecord *epics_record,
			const char *epics_field_name,
			int direction );

	caStatus write( const gdd &gdd_object );

    private:
	int direction;
	int home_value;
};

class mxcaMotorTweakPV : public mxcaPV {
    public:
	mxcaMotorTweakPV( mxcaRecord *epics_record,
			const char *epics_field_name,
			int direction );

	caStatus write( const gdd &gdd_object );

    private:
	int direction;
	short tweak_value;
};

class mxcaMotorRxpPV : public mxcaPV {
    public:
	mxcaMotorRxpPV( mxcaRecord *epics_record,
			const char *epics_field_name );

	caStatus read( void );

    private:
	double deadband;
	double position;
	double old_position;
};

class mxcaMotorMstaPV : public mxcaPV {
    friend class mxcaMotorDvalPV;
    friend class mxcaMotorHomePV;
    friend class mxcaMotorJogPV;

    public:
	mxcaMotorMstaPV( mxcaRecord *epics_record );

	caStatus read( void );

    private:
	MX_MOTOR *motor;
	MX_RECORD_FIELD *status_rf;
	mx_bool_type force_value_change;

	unsigned long msta_value;
	short athm_value;
	short dmov_value;
	short movn_value;
};

