/*
 * Name:    mxcaRecord.cpp
 *
 * Purpose: This file defines the mxcaRecord class and the classes that 
 *          inherit from it.
 *
 *-----------------------------------------------------------------------
 *
 * Copyright 2004, 2008-2009 Illinois Institute of Technology
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#define MXCA_DEBUG_RECORD	FALSE

#include <iostream>
#include <fstream>

#include "mxcaServer.h"

mxcaRecord::mxcaRecord( mxcaServer *epics_server,
			const char *epics_record_name,
			MX_RECORD *mx_record )
{
#if MXCA_DEBUG_RECORD
	static const char fname[] = "mxcaRecord::mxcaRecord()";

	MX_DEBUG(-2,("%s invoked for epics record '%s'",
		fname, epics_record_name ));
#endif

	mx_status_type mx_status;

	this->epicsServer = epics_server;

	this->mx_record = mx_record;

	mx_record->application_ptr = this;

	if ( epics_record_name != NULL ) {
		strlcpy( this->epicsRecordName, epics_record_name,
			sizeof( this->epicsRecordName ) );
	}

#if MXCA_DEBUG_RECORD
	MX_DEBUG( 2,("%s: record '%s' is a '%s'",
		fname, mx_record->name, mx_get_driver_name( mx_record ) ));
#endif

	if ( mx_record->mx_superclass != MXR_LIST_HEAD ) {
		mx_status = this->setupRecordPVs();
	}

#if MXCA_DEBUG_RECORD
	MX_DEBUG( 2,("%s complete for epics record '%s'",
		fname, epics_record_name ));
#endif
}

mx_status_type
mxcaRecord::setupRecordPVs( void )
{
#if MXCA_DEBUG_RECORD
	static const char fname[] = "mxcaServer::setupRecordPVs()";

	MX_DEBUG(-2,("%s invoked for '%s'", fname, this->mx_record->name));
#endif

	mxcaPV *pv;

	pv = new mxcaPV( this, "NAME" );

	pv->configure_mx_field_pv( "name" );

	pv = new mxcaPV( this, "DESC" );

	pv->configure_mx_field_pv( "label" );

#if MXCA_DEBUG_RECORD
	MX_DEBUG(-2,("%s complete for '%s'", fname, this->mx_record->name));
#endif

	return MX_SUCCESSFUL_RESULT;
}

const char *
mxcaRecord::get_epics_record_name( void )
{
	return this->epicsRecordName;
}

mxcaPV *
mxcaRecord::findPV( const char *field_name )
{
	mxcaPV *pv;
	char pvname[80];

	if ( this->epicsServer == (mxcaServer *) NULL ) {
		return NULL;
	}

	snprintf( pvname, sizeof(pvname), "%s.%s",
		this->epicsRecordName, field_name );

	pv = this->epicsServer->findPV( pvname );

	return pv;
}

