/*
 * Name:    mxcaScalerRecord.h
 *
 * Purpose: This header file defines the mxcaScalerRecord class.
 *
 *-----------------------------------------------------------------------
 *
 * Copyright 2009 Illinois Institute of Technology
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

class mxcaScalerCntPV;
class mxcaScalerFreqPV;
class mxcaScalerPr1PV;
class mxcaScalerS1PV;
class mxcaScalerTPV;
class mxcaScalerTpPV;

class mxcaScalerGnPV;
class mxcaScalerSnPV;

/*------------------------------------------------------------------------*/

#define MXCA_SCALER_MAX_INPUTS	15

typedef struct {
	MX_RECORD *mx_input_record;
	MX_RECORD_FIELD *mx_value_field;
	int channel_number;
	unsigned long preset_count;
	unsigned long value;

	mxcaScalerGnPV *gn_pv;
	mxcaScalerSnPV *sn_pv;
} mxcaInputInfoStruct;

/*------------------------------------------------------------------------*/

class mxcaScalerRecord : public mxcaRecord {
    friend class mxcaScalerCntPV;
    friend class mxcaScalerFreqPV;
    friend class mxcaScalerPr1PV;
    friend class mxcaScalerS1PV;
    friend class mxcaScalerTPV;
    friend class mxcaScalerTpPV;

    friend class mxcaScalerGnPV;
    friend class mxcaScalerPrnPV;
    friend class mxcaScalerSnPV;

    public:
    	mxcaScalerRecord ( mxcaServer *epics_server,
			const char *epics_record_name,
			MX_RECORD *mx_timer_record );

	mx_status_type add_input( MX_RECORD *mx_input_record );

	void update_number_of_channels( void );

    private:
	void post_gate_values_changed( void );

	double freq_value;
	unsigned long pr1_value;
	unsigned long s1_value;
	double t_value;
	double tp_value;

	int gate_channel;

	mxcaScalerCntPV *cnt_pv;
	mxcaScalerFreqPV *freq_pv;
	mxcaScalerGnPV *g1_pv;
	mxcaScalerPr1PV *pr1_pv;
	mxcaScalerS1PV *s1_pv;
	mxcaScalerTPV *t_pv;
	mxcaScalerTpPV *tp_pv;

	int num_inputs;
	mxcaInputInfoStruct input_array[MXCA_SCALER_MAX_INPUTS];
};

/*------------------------------------------------------------------------*/

class mxcaScalerCntPV : public mxcaEnumPV {
    public:
	mxcaScalerCntPV( mxcaScalerRecord *epics_scaler_record );

	caStatus read( void );
	caStatus write( const gdd &gdd_object );

};

class mxcaScalerFreqPV : public mxcaPV {
    public:
	mxcaScalerFreqPV( mxcaScalerRecord *epics_scaler_record );

	caStatus read( void );
	caStatus write( const gdd &gdd_object );
	mx_bool_type post_value_changed( void );
};

class mxcaScalerPr1PV : public mxcaPV {
    friend class mxcaScalerTpPV;

    public:
	mxcaScalerPr1PV( mxcaScalerRecord *epics_scaler_record );

	caStatus read( void );
	caStatus write( const gdd &gdd_object );
	mx_bool_type post_value_changed( void );
};

class mxcaScalerS1PV : public mxcaPV {
    public:
	mxcaScalerS1PV( mxcaScalerRecord *epics_scaler_record );

	caStatus read( void );
	caStatus write( const gdd &gdd_object );
	mx_bool_type post_value_changed( void );
};

class mxcaScalerTPV : public mxcaPV {
    friend class mxcaScalerFreqPV;
    friend class mxcaScalerS1PV;

    public:
	mxcaScalerTPV( mxcaScalerRecord *epics_scaler_record );

	caStatus read( void );
	caStatus write( const gdd &gdd_object );
};

class mxcaScalerTpPV : public mxcaPV {
    friend class mxcaScalerFreqPV;
    friend class mxcaScalerPr1PV;

    public:
	mxcaScalerTpPV( mxcaScalerRecord *epics_scaler_record );

	caStatus read( void );
	caStatus write( const gdd &gdd_object );
	mx_bool_type post_value_changed( void );
};

/*------------------------------------------------------------------------*/

class mxcaScalerGnPV : public mxcaEnumPV {
    public:
	mxcaScalerGnPV( mxcaScalerRecord *epics_scaler_record,
			const char *epics_field_name );

	caStatus read( void );
	caStatus write( const gdd &gdd_object );

    private:
	int channel_number;
};

class mxcaScalerPrnPV : public mxcaPV {
    public:
	mxcaScalerPrnPV( mxcaScalerRecord *epics_scaler_record,
			const char *epics_field_name );

	caStatus read( void );
	caStatus write( const gdd &gdd_object );

    private:
	int channel_number;
};

class mxcaScalerSnPV : public mxcaPV {
    friend class mxcaScalerCntPV;

    public:
	mxcaScalerSnPV( mxcaScalerRecord *epics_scaler_record,
			const char *epics_field_name );

	caStatus read( void );
	caStatus write( const gdd &gdd_object );

    private:
	int channel_number;
	mx_bool_type acquisition_in_progress;

	mxcaScalerCntPV *cnt_pv;
};

